@@ -17,10 +17,18 @@ namespace driver
     ze_dditable_driver_t pCore;
     zet_dditable_driver_t pTools;
     zes_dditable_driver_t pSysman;
+    zer_dditable_driver_t pRuntime;
 
     //////////////////////////////////////////////////////////////////////////
     context_t::context_t()
     {
+        auto ddi_test_disable = getenv_string( "ZEL_TEST_NULL_DRIVER_DISABLE_DDI_EXT" );
+        #ifndef ZEL_NULL_DRIVER_ID
+        #define ZEL_NULL_DRIVER_ID 1
+        #endif
+        std::string null_driver_id_str = std::to_string(ZEL_NULL_DRIVER_ID);
+        ddiExtensionSupported = (ddi_test_disable != null_driver_id_str && ddi_test_disable != "3");
+        
         zesDdiTable.Driver.pfnGet = [](
             uint32_t* pCount,
             ze_driver_handle_t* phDrivers )
@@ -68,15 +76,9 @@ namespace driver
         {
             auto pNext = reinterpret_cast<ze_base_properties_t *>(pDriverProperties->pNext);
             while (pNext) {
-                auto ddi_test_disable = getenv_string( "ZEL_TEST_NULL_DRIVER_DISABLE_DDI_EXT" );
-                #ifndef ZEL_NULL_DRIVER_ID
-                #define ZEL_NULL_DRIVER_ID 1
-                #endif
-                std::string null_driver_id_str = std::to_string(ZEL_NULL_DRIVER_ID);
-                if (pNext->stype == ZE_STRUCTURE_TYPE_DRIVER_DDI_HANDLES_EXT_PROPERTIES && (ddi_test_disable != null_driver_id_str && ddi_test_disable != "3")) {
+                if (pNext->stype == ZE_STRUCTURE_TYPE_DRIVER_DDI_HANDLES_EXT_PROPERTIES && context.ddiExtensionSupported) {
                     ze_driver_ddi_handles_ext_properties_t *pDdiHandlesExtProperties = reinterpret_cast<ze_driver_ddi_handles_ext_properties_t *>(pNext);
                     pDdiHandlesExtProperties->flags = ze_driver_ddi_handle_ext_flag_t::ZE_DRIVER_DDI_HANDLE_EXT_FLAG_DDI_HANDLE_EXT_SUPPORTED;
-                    context.ddiExtensionRequested = true;
                 }
                 pNext = reinterpret_cast<ze_base_properties_t *>(pNext->pNext);
             }
@@ -374,9 +376,22 @@ namespace driver
             ze_device_properties_t deviceProperties = {};
             deviceProperties.stype = ZE_STRUCTURE_TYPE_DEVICE_PROPERTIES;
             deviceProperties.type = ZE_DEVICE_TYPE_GPU;
+
+            // Check compile-time definitions first
+            #ifdef ZEL_NULL_DRIVER_TYPE_NPU
+            deviceProperties.type = ZE_DEVICE_TYPE_VPU;
+            #endif
+
+            #ifdef ZEL_NULL_DRIVER_TYPE_GPU
+            deviceProperties.type = ZE_DEVICE_TYPE_GPU;
+            #endif
+
+            // Environment variable can override
             auto driver_type = getenv_string( "ZEL_TEST_NULL_DRIVER_TYPE" );
             if (std::strcmp(driver_type.c_str(), "NPU") == 0) {
                 deviceProperties.type = ZE_DEVICE_TYPE_VPU;
+            } else if (std::strcmp(driver_type.c_str(), "GPU") == 0) {
+                deviceProperties.type = ZE_DEVICE_TYPE_GPU;
             }
 #if defined(_WIN32)
             strcpy_s( deviceProperties.name, "Null Device" );
@@ -418,6 +433,9 @@ namespace driver
             }
             *pCount = 1;
 
+            char *env_str = context.setenv_var_with_driver_id("zeDeviceGetCommandQueueGroupProperties", ZEL_NULL_DRIVER_ID);
+            context.env_vars.push_back(env_str);
+
             return ZE_RESULT_SUCCESS;
         };
 
@@ -433,6 +451,9 @@ namespace driver
             *pCount = 1;
             if( nullptr != pMemProperties ) *pMemProperties = memoryProperties;
 
+            char *env_str = context.setenv_var_with_driver_id("zeDeviceGetMemoryProperties", ZEL_NULL_DRIVER_ID);
+            context.env_vars.push_back(env_str);
+
             return ZE_RESULT_SUCCESS;
         };
 
@@ -445,6 +466,10 @@ namespace driver
             memoryAccessProperties.stype = ZE_STRUCTURE_TYPE_DEVICE_MEMORY_ACCESS_PROPERTIES;
 
             *pMemAccessProperties = memoryAccessProperties;
+            
+            char *env_str = context.setenv_var_with_driver_id("zeDeviceGetMemoryAccessProperties", ZEL_NULL_DRIVER_ID);
+            context.env_vars.push_back(env_str);
+
             return ZE_RESULT_SUCCESS;
         };
 
@@ -486,17 +511,32 @@ namespace driver
         {
             if( nullptr != pExtensionProperties )
             {
-                ze_driver_extension_properties_t driverExtensionProperties = {};
+                ze_driver_extension_properties_t tracingExtension = {};
 #if defined(_WIN32)
-                strcpy_s( driverExtensionProperties.name, ZET_API_TRACING_EXP_NAME );
+                strcpy_s( tracingExtension.name, ZET_API_TRACING_EXP_NAME );
 #else
-                strcpy( driverExtensionProperties.name, ZET_API_TRACING_EXP_NAME );
+                strcpy( tracingExtension.name, ZET_API_TRACING_EXP_NAME );
+#endif
+                tracingExtension.version = ZET_API_TRACING_EXP_VERSION_1_0;
+                pExtensionProperties[0] = tracingExtension;
+                
+                ze_driver_extension_properties_t ddiHandlesExtension = {};
+#if defined(_WIN32)
+                strcpy_s( ddiHandlesExtension.name, ZE_DRIVER_DDI_HANDLES_EXT_NAME );
+#else
+                strcpy( ddiHandlesExtension.name, ZE_DRIVER_DDI_HANDLES_EXT_NAME );
 #endif
-                driverExtensionProperties.version = ZET_API_TRACING_EXP_VERSION_1_0;
 
-                *pExtensionProperties = driverExtensionProperties;
+                auto ddi_version_env = getenv_string("ZEL_TEST_DDI_HANDLES_EXT_VERSION");
+                if (!ddi_version_env.empty() && ddi_version_env == "1_0") {
+                    ddiHandlesExtension.version = ZE_DRIVER_DDI_HANDLES_EXT_VERSION_1_0;
+                } else {
+                    ddiHandlesExtension.version = ZE_DRIVER_DDI_HANDLES_EXT_VERSION_1_1;
+                }
+                
+                pExtensionProperties[1] = ddiHandlesExtension;
             }
-            *pCount = 1;
+            *pCount = 2;
 
             return ZE_RESULT_SUCCESS;
         };
@@ -628,6 +668,16 @@ namespace driver
         pSysman.Driver = &zesDdiTable.Driver;
         pSysman.isValidFlag = 1;
         pSysman.version = ZE_API_VERSION_CURRENT;
+
+        static zer_global_dditable_t runtimeDdiTable;
+        runtimeDdiTable.pfnGetLastErrorDescription = driver::zerGetLastErrorDescription;
+        runtimeDdiTable.pfnTranslateDeviceHandleToIdentifier = driver::zerTranslateDeviceHandleToIdentifier;
+        runtimeDdiTable.pfnTranslateIdentifierToDeviceHandle = driver::zerTranslateIdentifierToDeviceHandle;
+        runtimeDdiTable.pfnGetDefaultContext = driver::zerGetDefaultContext;
+
+        pRuntime.Global = &runtimeDdiTable;
+        pRuntime.isValidFlag = 1;
+        pRuntime.version = ZE_API_VERSION_CURRENT;
     }
 
     char *context_t::setenv_var_with_driver_id(const std::string &key, uint32_t driverId)
